const winston = require('winston');
const WinstonCloudWatch = require('winston-cloudwatch');
const sanitizer = require('node-sanitizer');
const { Client } = require('@elastic/elasticsearch');
const uuidv4 = require('uuid/v4');
const {
  env, serviceName, elasticLog, sanitizedFields
} = require('@config/vars');

const client = new Client({
  cloud: {
    id: elasticLog.cloudId,
    username: elasticLog.username,
    password: elasticLog.password
  }
});

const {
  combine,
  colorize,
  simple
} = winston.format;

const options = {
  console: {
    level: 'info',
    handleExceptions: true,
    json: false,
    colorize: true,
    prettyPrint: true,
    format: combine(
      colorize(),
      simple()
    )
  }
};

const transports = [
  new winston.transports.Console(options.console),
  new WinstonCloudWatch({
    logGroupName: serviceName,
    logStreamName: `${process.env.NODE_ENV}-${process.env.POD_NAME}`,
    createLogGroup: true,
    createLogStream: true,
    handleExceptions: true,
    jsonMessage: true,
    awsRegion: process.env.AWS_DEFAULT_REGION
  })
];

// instantiate a new Winston Logger with the settings defined above
const logger = winston.loggers.add(env, {
  transports
});

// create a stream object with a 'write' function that will be used by `loggerMiddleware`
logger.stream = {
  write: (req) => {
    logger.info(`${req.method} request to ${req.url}`, {
      body: sanitizer(req.body, sanitizedFields),
      query: sanitizer(req.query, sanitizedFields)
    });
  }
};

logger.streamError = {
  write: (req) => {
    logger.error(`${req.method} request to ${req.url}`, {
      body: sanitizer(req.body, sanitizedFields),
      query: sanitizer(req.query, sanitizedFields)
    });
  }
};

/**
 * Function to send data to elastic search
 * @param {object} body         Request Payload
 */
/* istanbul ignore next */
const sendDataToElastic = async (body) => {
  if (env === 'test') {
    return;
  }
  const date = new Date();
  const month = (`0${date.getMonth() + 1}`).slice(-2);
  const year = date.getFullYear();
  const day = (`0${date.getDate()}`).slice(-2);
  client.create({
    id: uuidv4(),
    index: `${serviceName}-${process.env.NODE_ENV}-${year}.${month}.${day}`,
    body: { ...body, timestamp: new Date() }
  }, (err) => {
    if (err) logger.error('ES Push Error', { msg: err.message });
  });
};

/**
 * Debug Log
 * @param {String} message    Info message
 * @param {Object} logData       Info data
 */
const debug = (message, logData) => {
  logger.info(message, logData);
  // sendDataToElastic({ message, ...logData });
};

/**
 * Warn Log
 * @param {String} message       Error message
 * @param {Object} logData       Error data
 */
const warn = (message, logData) => {
  logger.error(message, logData);
  // sendDataToElastic({ message, ...logData });
};

/**
 * Info Log
 * @param {String} message    Info message
 * @param {Object} logData       Info data
 */
const info = (message, logData) => {
  logger.info(message, logData);
  // sendDataToElastic({ message, ...logData });
};

/**
 * Error Log
 * @param {String} message       Error message
 * @param {Object} logData       Error data
 */
const error = (message, logData) => {
  logger.error(message, logData);
  // sendDataToElastic({ message, ...logData });
};

/**
 * Capture Error
 * @param {String} title        Error's title
 * @param {Object} error        Error's data
 * @param {String} methodName   Method's name
 */
const captureError = (title, err, methodName) => {
  const logData = {
    serviceName,
    methodName,
    error: err.message
  };
  if (err.stack) {
    logData.stack = err.stack;
  }
  logger.error(title, logData);
};

module.exports = {
  logger,
  info,
  debug,
  warn,
  error,
  captureError,
  sendDataToElastic
};
